package com.sun.media.sound;

import java.io.DataOutputStream;
import javax.sound.sampled.AudioFormat;
import java.io.DataInputStream;
import javax.sound.sampled.AudioInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.net.URL;
import java.io.IOException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.sound.sampled.AudioFileFormat;
import java.io.InputStream;

public final class AiffFileReader extends SunFileReader
{
    private static final int MAX_READ_LENGTH = 8;
    
    public AiffFileReader() {
    }
    
    @Override
    public AudioFileFormat getAudioFileFormat(InputStream inputStream) throws UnsupportedAudioFileException, IOException {
        AudioFileFormat comm = this.getCOMM(inputStream, true);
        inputStream.reset();
        return comm;
    }
    
    @Override
    public AudioFileFormat getAudioFileFormat(URL url) throws UnsupportedAudioFileException, IOException {
        AudioFileFormat comm = null;
        InputStream openStream = url.openStream();
        try {
            comm = this.getCOMM(openStream, false);
        }
        finally {
            openStream.close();
        }
        return comm;
    }
    
    @Override
    public AudioFileFormat getAudioFileFormat(File file) throws UnsupportedAudioFileException, IOException {
        AudioFileFormat comm = null;
        FileInputStream fileInputStream = new FileInputStream(file);
        try {
            comm = this.getCOMM(fileInputStream, false);
        }
        finally {
            fileInputStream.close();
        }
        return comm;
    }
    
    @Override
    public AudioInputStream getAudioInputStream(InputStream stream) throws UnsupportedAudioFileException, IOException {
        AudioFileFormat comm = this.getCOMM(stream, true);
        return new AudioInputStream(stream, comm.getFormat(), (long)comm.getFrameLength());
    }
    
    @Override
    public AudioInputStream getAudioInputStream(URL url) throws UnsupportedAudioFileException, IOException {
        InputStream openStream = url.openStream();
        AudioFileFormat comm = null;
        try {
            comm = this.getCOMM(openStream, false);
        }
        finally {
            if (comm == null) {
                openStream.close();
            }
        }
        return new AudioInputStream(openStream, comm.getFormat(), (long)comm.getFrameLength());
    }
    
    @Override
    public AudioInputStream getAudioInputStream(File file) throws UnsupportedAudioFileException, IOException {
        FileInputStream stream = new FileInputStream(file);
        AudioFileFormat comm = null;
        try {
            comm = this.getCOMM(stream, false);
        }
        finally {
            if (comm == null) {
                stream.close();
            }
        }
        return new AudioInputStream(stream, comm.getFormat(), (long)comm.getFrameLength());
    }
    
    private AudioFileFormat getCOMM(InputStream in, boolean b) throws UnsupportedAudioFileException, IOException {
        DataInputStream dataInputStream = new DataInputStream(in);
        if (b) {
            dataInputStream.mark(8);
        }
        int n = 0;
        int n2 = 0;
        Object o = null;
        if (dataInputStream.readInt() != 1179603533) {
            if (b) {
                dataInputStream.reset();
            }
            throw new UnsupportedAudioFileException("not an AIFF file");
        }
        else {
            int int1 = dataInputStream.readInt();
            int int2 = dataInputStream.readInt();
            n += 12;
            int n3;
            if (int1 <= 0) {
                int1 = -1;
                n3 = -1;
            }
            else {
                n3 = int1 + 8;
            }
            boolean n4 = false;
            if (int2 == 1095321155) {
                n4 = true;
            }
            int i = 0;
            while (i == 0) {
                int int3 = dataInputStream.readInt();
                int int4 = dataInputStream.readInt();
                n += 8;
                int n5 = 0;
                switch (int3) {
                    case 1180058962: {
                        break;
                    }
                    case 1129270605: {
                        if ((n4 == 0 && int4 < 18) || (n4 && int4 < 22)) {
                            throw new UnsupportedAudioFileException("Invalid AIFF/COMM chunksize");
                        }
                        int unsignedShort = dataInputStream.readUnsignedShort();
                        if (unsignedShort <= 0) {
                            throw new UnsupportedAudioFileException("Invalid number of channels");
                        }
                        dataInputStream.readInt();
                        int unsignedShort2 = dataInputStream.readUnsignedShort();
                        if (unsignedShort2 < 1 || unsignedShort2 > 32) {
                            throw new UnsupportedAudioFileException("Invalid AIFF/COMM sampleSize");
                        }
                        float n6 = (float)this.read_ieee_extended(dataInputStream);
                        n5 += 18;
                        AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
                        if (!(n4 == 0)) {
                            int int5 = dataInputStream.readInt();
                            n5 += 4;
                            switch (int5) {
                                case 1313820229: {
                                    encoding = AudioFormat.Encoding.PCM_SIGNED;
                                    break;
                                }
                                case 1970037111: {
                                    encoding = AudioFormat.Encoding.ULAW;
                                    unsignedShort2 = 8;
                                    break;
                                }
                                default: {
                                    throw new UnsupportedAudioFileException("Invalid AIFF encoding");
                                }
                            }
                        }
                        o = new AudioFormat(encoding, n6, unsignedShort2, unsignedShort, SunFileReader.calculatePCMFrameSize(unsignedShort2, unsignedShort), n6, true);
                        break;
                    }
                    case 1397968452: {
                        dataInputStream.readInt();
                        dataInputStream.readInt();
                        n5 += 8;
                        if (int4 < int1) {
                            n2 = int4 - n5;
                        }
                        else {
                            n2 = int1 - (n + n5);
                        }
                        i = 1;
                        break;
                    }
                }
                n += n5;
                if (i == 0) {
                    int n7 = int4 - n5;
                    if (n7 > 0) {
                        n += dataInputStream.skipBytes(n7);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            if (o == null) {
                throw new UnsupportedAudioFileException("missing COMM chunk");
            }
            return new AiffFileFormat(n4 ? AudioFileFormat.Type.AIFC : AudioFileFormat.Type.AIFF, n3, o, n2 / o.getFrameSize());
        }
    }
    
    private void write_ieee_extended(DataOutputStream dataOutputStream, double n) throws IOException {
        int v;
        for (v = 16398, ; n < 44000.0; n *= 2.0, --v) {}
        dataOutputStream.writeShort(v);
        dataOutputStream.writeInt((int)n << 16);
        dataOutputStream.writeInt(0);
    }
    
    private double read_ieee_extended(DataInputStream dataInputStream) throws IOException {
        double n = 3.4028234663852886E38;
        int unsignedShort = dataInputStream.readUnsignedShort();
        long n2 = (long)dataInputStream.readUnsignedShort() << 16 | (long)dataInputStream.readUnsignedShort();
        long n3 = (long)dataInputStream.readUnsignedShort() << 16 | (long)dataInputStream.readUnsignedShort();
        double n5;
        if (unsignedShort == 0 && n2 == 0L && n3 == 0L) {
            n5 = 0.0;
        }
        else if (unsignedShort == 32767) {
            n5 = n;
        }
        else {
            unsignedShort -= 16383;
            unsignedShort -= 31;
            double n4 = (double)n2 * Math.pow(2.0, (double)unsignedShort);
            unsignedShort -= 32;
            n5 = n4 + (double)n3 * Math.pow(2.0, (double)unsignedShort);
        }
        return n5;
    }
}
